@@ -45,13 +45,12 @@
 #endif
 
 DEFINE_GUID(KSDATAFORMAT_SUBTYPE_PCM, 0x00000001, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71);
+DEFINE_GUID(KSDATAFORMAT_SUBTYPE_IEEE_FLOAT, 0x00000003, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71);
 
 static void *ds_handle;
 static HRESULT (WINAPI *pDirectSoundCreate)(LPCGUID pcGuidDevice, LPDIRECTSOUND *ppDS, LPUNKNOWN pUnkOuter);
 static HRESULT (WINAPI *pDirectSoundEnumerateA)(LPDSENUMCALLBACKA pDSEnumCallback, LPVOID pContext);
 
-// Since DSound doesn't report the fragment size, emulate it
-static int num_frags;
 
 typedef struct {
     // DirectSound Playback Device
@@ -68,7 +67,10 @@ typedef struct {
     ALCchar *name;
     GUID guid;
 } DevMap;
-static DevMap DeviceList[16];
+
+static const ALCchar dsDevice[] = "DirectSound Software";
+static DevMap *DeviceList;
+static ALuint NumDevices;
 
 
 static ALuint DSoundProc(ALvoid *ptr)
@@ -80,6 +82,7 @@ static ALuint DSoundProc(ALvoid *ptr)
     DWORD PlayCursor;
     VOID *WritePtr1, *WritePtr2;
     DWORD WriteCnt1,  WriteCnt2;
+    DWORD FrameSize;
     DWORD FragSize;
     DWORD avail;
     HRESULT err;
@@ -90,9 +93,13 @@ static ALuint DSoundProc(ALvoid *ptr)
     if(FAILED(err))
     {
         AL_PRINT("Failed to get buffer caps: 0x%lx\n", err);
+        aluHandleDisconnect(pDevice);
         return 1;
     }
-    FragSize = DSBCaps.dwBufferBytes / num_frags;
+
+    FrameSize = aluChannelsFromFormat(pDevice->Format) *
+                aluBytesFromFormat(pDevice->Format);
+    FragSize = pDevice->UpdateSize * FrameSize;
 
     IDirectSoundBuffer_GetCurrentPosition(pData->DSsbuffer, &LastCursor, NULL);
     while(!pData->killNow)
@@ -127,10 +134,8 @@ static ALuint DSoundProc(ALvoid *ptr)
         if(SUCCEEDED(err))
         {
             // If we have an active context, mix data directly into output buffer otherwise fill with silence
-            SuspendContext(NULL);
-            aluMixData(pDevice->Context, WritePtr1, WriteCnt1, pDevice->Format);
-            aluMixData(pDevice->Context, WritePtr2, WriteCnt2, pDevice->Format);
-            ProcessContext(NULL);
+            aluMixData(pDevice, WritePtr1, WriteCnt1/FrameSize);
+            aluMixData(pDevice, WritePtr2, WriteCnt2/FrameSize);
 
             // Unlock output buffer only when successfully locked
             IDirectSoundBuffer_Unlock(pData->DSsbuffer, WritePtr1, WriteCnt1, WritePtr2, WriteCnt2);
@@ -148,38 +153,29 @@ static ALuint DSoundProc(ALvoid *ptr)
 
 static ALCboolean DSoundOpenPlayback(ALCdevice *device, const ALCchar *deviceName)
 {
-    DSBUFFERDESC DSBDescription;
     DSoundData *pData = NULL;
-    WAVEFORMATEXTENSIBLE OutputType;
-    DWORD frameSize = 0;
     LPGUID guid = NULL;
-    ALenum format = 0;
-    DWORD speakers;
     HRESULT hr;
 
     if(ds_handle == NULL)
         return ALC_FALSE;
 
-    if(deviceName)
+    if(!deviceName)
+        deviceName = dsDevice;
+    else if(strcmp(deviceName, dsDevice) != 0)
     {
-        int i;
-        for(i = 0;DeviceList[i].name;i++)
+        ALuint i;
+        for(i = 0;i < NumDevices;i++)
         {
             if(strcmp(deviceName, DeviceList[i].name) == 0)
             {
-                device->szDeviceName = DeviceList[i].name;
-                if(i > 0)
-                    guid = &DeviceList[i].guid;
+                guid = &DeviceList[i].guid;
                 break;
             }
         }
-        if(!DeviceList[i].name)
+        if(i == NumDevices)
             return ALC_FALSE;
     }
-    else
-        device->szDeviceName = DeviceList[0].name;
-
-    memset(&OutputType, 0, sizeof(OutputType));
 
     //Initialise requested device
 
@@ -194,25 +190,57 @@ static ALCboolean DSoundOpenPlayback(ALCdevice *device, const ALCchar *deviceNam
     hr = pDirectSoundCreate(guid, &pData->lpDS, NULL);
     if(SUCCEEDED(hr))
         hr = IDirectSound_SetCooperativeLevel(pData->lpDS, GetForegroundWindow(), DSSCL_PRIORITY);
-
-    if(SUCCEEDED(hr))
+    if(FAILED(hr))
     {
-        if(*(GetConfigValue(NULL, "format", "")) == 0)
-            hr = IDirectSound_GetSpeakerConfig(pData->lpDS, &speakers);
+        if(pData->lpDS)
+            IDirectSound_Release(pData->lpDS);
+        free(pData);
+        return ALC_FALSE;
+    }
+
+    device->szDeviceName = strdup(deviceName);
+    device->ExtraData = pData;
+    return ALC_TRUE;
+}
+
+static void DSoundClosePlayback(ALCdevice *device)
+{
+    DSoundData *pData = device->ExtraData;
+
+    IDirectSound_Release(pData->lpDS);
+    free(pData);
+    device->ExtraData = NULL;
+}
+
+static ALCboolean DSoundResetPlayback(ALCdevice *device)
+{
+    DSoundData *pData = (DSoundData*)device->ExtraData;
+    DSBUFFERDESC DSBDescription;
+    WAVEFORMATEXTENSIBLE OutputType;
+    DWORD frameSize = 0;
+    ALenum format = 0;
+    DWORD speakers;
+    HRESULT hr;
+
+    memset(&OutputType, 0, sizeof(OutputType));
+
+    hr = IDirectSound_GetSpeakerConfig(pData->lpDS, &speakers);
+    if(SUCCEEDED(hr) && *(GetConfigValue(NULL, "format", "")) != 0)
+    {
+        if(aluChannelsFromFormat(device->Format) == 1)
+            speakers = DSSPEAKER_COMBINED(DSSPEAKER_MONO, 0);
+        else if(aluChannelsFromFormat(device->Format) == 2)
+            speakers = DSSPEAKER_COMBINED(DSSPEAKER_STEREO, 0);
+        else if(aluChannelsFromFormat(device->Format) == 4)
+            speakers = DSSPEAKER_COMBINED(DSSPEAKER_QUAD, 0);
+        else if(aluChannelsFromFormat(device->Format) == 6)
+            speakers = DSSPEAKER_COMBINED(DSSPEAKER_5POINT1, 0);
+        else if(aluChannelsFromFormat(device->Format) == 8)
+            speakers = DSSPEAKER_COMBINED(DSSPEAKER_7POINT1, 0);
         else
         {
-            if(device->Format == AL_FORMAT_MONO8 || device->Format == AL_FORMAT_MONO16)
-                speakers = DSSPEAKER_COMBINED(DSSPEAKER_MONO, 0);
-            else if(device->Format == AL_FORMAT_STEREO8 || device->Format == AL_FORMAT_STEREO16)
-                speakers = DSSPEAKER_COMBINED(DSSPEAKER_STEREO, 0);
-            else if(device->Format == AL_FORMAT_QUAD8 || device->Format == AL_FORMAT_QUAD16)
-                speakers = DSSPEAKER_COMBINED(DSSPEAKER_QUAD, 0);
-            else if(device->Format == AL_FORMAT_51CHN8 || device->Format == AL_FORMAT_51CHN16)
-                speakers = DSSPEAKER_COMBINED(DSSPEAKER_5POINT1, 0);
-            else if(device->Format == AL_FORMAT_71CHN8 || device->Format == AL_FORMAT_71CHN16)
-                speakers = DSSPEAKER_COMBINED(DSSPEAKER_7POINT1, 0);
-            else
-                hr = IDirectSound_GetSpeakerConfig(pData->lpDS, &speakers);
+            AL_PRINT("Unknown format: 0x%x\n", device->Format);
+            return ALC_FALSE;
         }
     }
     if(SUCCEEDED(hr))
@@ -222,22 +250,28 @@ static ALCboolean DSoundOpenPlayback(ALCdevice *device, const ALCchar *deviceNam
         {
             if(aluBytesFromFormat(device->Format) == 1)
                 format = AL_FORMAT_MONO8;
-            else
+            else if(aluBytesFromFormat(device->Format) == 2)
                 format = AL_FORMAT_MONO16;
+            else if(aluBytesFromFormat(device->Format) == 4)
+                format = AL_FORMAT_MONO_FLOAT32;
         }
         else if(speakers == DSSPEAKER_STEREO)
         {
             if(aluBytesFromFormat(device->Format) == 1)
                 format = AL_FORMAT_STEREO8;
-            else
+            else if(aluBytesFromFormat(device->Format) == 2)
                 format = AL_FORMAT_STEREO16;
+            else if(aluBytesFromFormat(device->Format) == 4)
+                format = AL_FORMAT_STEREO_FLOAT32;
         }
         else if(speakers == DSSPEAKER_QUAD)
         {
             if(aluBytesFromFormat(device->Format) == 1)
                 format = AL_FORMAT_QUAD8;
-            else
+            else if(aluBytesFromFormat(device->Format) == 2)
                 format = AL_FORMAT_QUAD16;
+            else if(aluBytesFromFormat(device->Format) == 4)
+                format = AL_FORMAT_QUAD32;
             OutputType.dwChannelMask = SPEAKER_FRONT_LEFT |
                                        SPEAKER_FRONT_RIGHT |
                                        SPEAKER_BACK_LEFT |
@@ -247,8 +281,10 @@ static ALCboolean DSoundOpenPlayback(ALCdevice *device, const ALCchar *deviceNam
         {
             if(aluBytesFromFormat(device->Format) == 1)
                 format = AL_FORMAT_51CHN8;
-            else
+            else if(aluBytesFromFormat(device->Format) == 2)
                 format = AL_FORMAT_51CHN16;
+            else if(aluBytesFromFormat(device->Format) == 4)
+                format = AL_FORMAT_51CHN32;
             OutputType.dwChannelMask = SPEAKER_FRONT_LEFT |
                                        SPEAKER_FRONT_RIGHT |
                                        SPEAKER_FRONT_CENTER |
@@ -260,8 +296,10 @@ static ALCboolean DSoundOpenPlayback(ALCdevice *device, const ALCchar *deviceNam
         {
             if(aluBytesFromFormat(device->Format) == 1)
                 format = AL_FORMAT_71CHN8;
-            else
+            else if(aluBytesFromFormat(device->Format) == 2)
                 format = AL_FORMAT_71CHN16;
+            else if(aluBytesFromFormat(device->Format) == 4)
+                format = AL_FORMAT_71CHN32;
             OutputType.dwChannelMask = SPEAKER_FRONT_LEFT |
                                        SPEAKER_FRONT_RIGHT |
                                        SPEAKER_FRONT_CENTER |
@@ -284,12 +322,15 @@ static ALCboolean DSoundOpenPlayback(ALCdevice *device, const ALCchar *deviceNam
         OutputType.Format.cbSize = 0;
     }
 
-    if(OutputType.Format.nChannels > 2)
+    if(OutputType.Format.nChannels > 2 || OutputType.Format.wBitsPerSample > 16)
     {
         OutputType.Format.wFormatTag = WAVE_FORMAT_EXTENSIBLE;
         OutputType.Samples.wValidBitsPerSample = OutputType.Format.wBitsPerSample;
         OutputType.Format.cbSize = 22;
-        OutputType.SubFormat = KSDATAFORMAT_SUBTYPE_PCM;
+        if(OutputType.Format.wBitsPerSample == 32)
+            OutputType.SubFormat = KSDATAFORMAT_SUBTYPE_IEEE_FLOAT;
+        else
+            OutputType.SubFormat = KSDATAFORMAT_SUBTYPE_PCM;
     }
     else
     {
@@ -309,7 +350,7 @@ static ALCboolean DSoundOpenPlayback(ALCdevice *device, const ALCchar *deviceNam
         memset(&DSBDescription,0,sizeof(DSBUFFERDESC));
         DSBDescription.dwSize=sizeof(DSBUFFERDESC);
         DSBDescription.dwFlags=DSBCAPS_GLOBALFOCUS|DSBCAPS_GETCURRENTPOSITION2;
-        DSBDescription.dwBufferBytes=(device->UpdateSize/num_frags) * num_frags * frameSize;
+        DSBDescription.dwBufferBytes=device->UpdateSize * device->NumUpdates * frameSize;
         DSBDescription.lpwfxFormat=&OutputType.Format;
         hr = IDirectSound_CreateSoundBuffer(pData->lpDS, &DSBDescription, &pData->DSsbuffer, NULL);
     }
@@ -329,45 +370,41 @@ static ALCboolean DSoundOpenPlayback(ALCdevice *device, const ALCchar *deviceNam
     {
         if (pData->DSsbuffer)
             IDirectSoundBuffer_Release(pData->DSsbuffer);
+        pData->DSsbuffer = NULL;
         if (pData->DSpbuffer)
             IDirectSoundBuffer_Release(pData->DSpbuffer);
-        if (pData->lpDS)
-            IDirectSound_Release(pData->lpDS);
-
-        free(pData);
+        pData->DSpbuffer = NULL;
         return ALC_FALSE;
     }
 
     device->Format = format;
-    device->UpdateSize /= num_frags;
 
     return ALC_TRUE;
 }
 
-static void DSoundClosePlayback(ALCdevice *device)
+static void DSoundStopPlayback(ALCdevice *device)
 {
     DSoundData *pData = device->ExtraData;
 
+    if(!pData->thread)
+        return;
+
     pData->killNow = 1;
     StopThread(pData->thread);
+    pData->thread = NULL;
 
     IDirectSoundBuffer_Release(pData->DSsbuffer);
+    pData->DSsbuffer = NULL;
     if (pData->DSpbuffer)
         IDirectSoundBuffer_Release(pData->DSpbuffer);
-    IDirectSound_Release(pData->lpDS);
-
-    free(pData);
-    device->ExtraData = NULL;
+    pData->DSpbuffer = NULL;
 }
 
 
-static ALCboolean DSoundOpenCapture(ALCdevice *pDevice, const ALCchar *deviceName, ALCuint frequency, ALCenum format, ALCsizei SampleSize)
+static ALCboolean DSoundOpenCapture(ALCdevice *pDevice, const ALCchar *deviceName)
 {
     (void)pDevice;
     (void)deviceName;
-    (void)frequency;
-    (void)format;
-    (void)SampleSize;
     return ALC_FALSE;
 }
 
@@ -403,6 +440,8 @@ static ALCuint DSoundAvailableSamples(ALCdevice *pDevice)
 BackendFuncs DSoundFuncs = {
     DSoundOpenPlayback,
     DSoundClosePlayback,
+    DSoundResetPlayback,
+    DSoundStopPlayback,
     DSoundOpenCapture,
     DSoundCloseCapture,
     DSoundStartCapture,
@@ -413,29 +452,34 @@ BackendFuncs DSoundFuncs = {
 
 static BOOL CALLBACK DSoundEnumDevices(LPGUID guid, LPCSTR desc, LPCSTR drvname, LPVOID data)
 {
-    size_t *iter = data;
+    (void)data;
     (void)drvname;
 
     if(guid)
     {
         char str[128];
-        snprintf(str, sizeof(str), "DirectSound Software on %s", desc);
-        DeviceList[*iter].name = AppendAllDeviceList(str);
-        DeviceList[*iter].guid = *guid;
-        (*iter)++;
+        void *temp;
+
+        temp = realloc(DeviceList, sizeof(DevMap) * (NumDevices+1));
+        if(temp)
+        {
+            DeviceList = temp;
+
+            snprintf(str, sizeof(str), "DirectSound Software on %s", desc);
+            AppendAllDeviceList(str);
+
+            DeviceList[NumDevices].name = strdup(str);
+            DeviceList[NumDevices].guid = *guid;
+            NumDevices++;
+        }
     }
-    else
-        DeviceList[0].name = AppendDeviceList("DirectSound Software");
 
     return TRUE;
 }
 
 void alcDSoundInit(BackendFuncs *FuncList)
 {
-    size_t iter = 1;
-    HRESULT hr;
-
-    *FuncList = DSoundFuncs;
+    if(FuncList) *FuncList = DSoundFuncs;
 
 #ifdef _WIN32
     ds_handle = LoadLibraryA("dsound.dll");
@@ -463,11 +507,45 @@ void alcDSoundInit(BackendFuncs *FuncList)
 LOAD_FUNC(DirectSoundCreate);
 LOAD_FUNC(DirectSoundEnumerateA);
 #undef LOAD_FUNC
+}
 
-    num_frags = GetConfigValueInt("dsound", "periods", 4);
-    if(num_frags < 2) num_frags = 2;
+void alcDSoundDeinit(void)
+{
+    ALuint i;
 
-    hr = pDirectSoundEnumerateA(DSoundEnumDevices, &iter);
-    if(FAILED(hr))
-        AL_PRINT("Error enumerating DirectSound devices (%#x)!\n", (unsigned int)hr);
+    for(i = 0;i < NumDevices;++i)
+        free(DeviceList[i].name);
+    free(DeviceList);
+    DeviceList = NULL;
+    NumDevices = 0;
+
+#ifdef _WIN32
+    if(ds_handle)
+        FreeLibrary(ds_handle);
+#endif
+    ds_handle = NULL;
+}
+
+void alcDSoundProbe(int type)
+{
+    if(!ds_handle) alcDSoundInit(NULL);
+    if(!ds_handle) return;
+
+    if(type == DEVICE_PROBE)
+        AppendDeviceList(dsDevice);
+    else if(type == ALL_DEVICE_PROBE)
+    {
+        HRESULT hr;
+        ALuint i;
+
+        for(i = 0;i < NumDevices;++i)
+            free(DeviceList[i].name);
+        free(DeviceList);
+        DeviceList = NULL;
+        NumDevices = 0;
+
+        hr = pDirectSoundEnumerateA(DSoundEnumDevices, NULL);
+        if(FAILED(hr))
+            AL_PRINT("Error enumerating DirectSound devices (%#x)!\n", (unsigned int)hr);
+    }
 }
