@@ -47,8 +47,8 @@
 #define SOUND_MIXER_WRITE MIXER_WRITE
 #endif
 
-static char *oss_device;
-static char *oss_device_capture;
+static const ALCchar oss_device[] = "OSS Software";
+static const ALCchar oss_device_capture[] = "OSS Capture";
 
 typedef struct {
     int fd;
@@ -79,17 +79,18 @@ static ALuint OSSProc(ALvoid *ptr)
 {
     ALCdevice *pDevice = (ALCdevice*)ptr;
     oss_data *data = (oss_data*)pDevice->ExtraData;
+    ALint frameSize;
     int wrote;
 
-    while(!data->killNow)
+    frameSize = aluChannelsFromFormat(pDevice->Format) *
+                aluBytesFromFormat(pDevice->Format);
+
+    while(!data->killNow && !pDevice->Connected)
     {
         ALint len = data->data_size;
         ALubyte *WritePtr = data->mix_data;
 
-        SuspendContext(NULL);
-        aluMixData(pDevice->Context, WritePtr, len, pDevice->Format);
-        ProcessContext(NULL);
-
+        aluMixData(pDevice, WritePtr, len/frameSize);
         while(len > 0 && !data->killNow)
         {
             wrote = write(data->fd, WritePtr, len);
@@ -98,6 +99,7 @@ static ALuint OSSProc(ALvoid *ptr)
                 if(errno != EAGAIN && errno != EWOULDBLOCK)
                 {
                     AL_PRINT("write failed: %s\n", strerror(errno));
+                    aluHandleDisconnect(pDevice);
                     len = 0;
                 }
                 else
@@ -129,6 +131,7 @@ static ALuint OSSCaptureProc(ALvoid *ptr)
         if(amt < 0)
         {
             AL_PRINT("read failed: %s\n", strerror(errno));
+            aluHandleDisconnect(pDevice);
             break;
         }
         if(amt == 0)
@@ -145,29 +148,15 @@ static ALuint OSSCaptureProc(ALvoid *ptr)
 
 static ALCboolean oss_open_playback(ALCdevice *device, const ALCchar *deviceName)
 {
-    int numFragmentsLogSize;
-    int log2FragmentSize;
-    unsigned int periods;
-    audio_buf_info info;
-    ALuint frameSize;
     char driver[64];
-    int numChannels;
     oss_data *data;
-    int ossFormat;
-    int ossSpeed;
-    char *err;
-    int i;
 
     strncpy(driver, GetConfigValue("oss", "device", "/dev/dsp"), sizeof(driver)-1);
     driver[sizeof(driver)-1] = 0;
-    if(deviceName)
-    {
-        if(strcmp(deviceName, oss_device))
-            return ALC_FALSE;
-        device->szDeviceName = oss_device;
-    }
-    else
-        device->szDeviceName = oss_device;
+    if(!deviceName)
+        deviceName = oss_device;
+    else if(strcmp(deviceName, oss_device) != 0)
+        return ALC_FALSE;
 
     data = (oss_data*)calloc(1, sizeof(oss_data));
     data->killNow = 0;
@@ -180,31 +169,71 @@ static ALCboolean oss_open_playback(ALCdevice *device, const ALCchar *deviceName
         return ALC_FALSE;
     }
 
+    device->szDeviceName = strdup(deviceName);
+    device->ExtraData = data;
+    return ALC_TRUE;
+}
+
+static void oss_close_playback(ALCdevice *device)
+{
+    oss_data *data = (oss_data*)device->ExtraData;
+
+    close(data->fd);
+    free(data);
+    device->ExtraData = NULL;
+}
+
+static ALCboolean oss_reset_playback(ALCdevice *device)
+{
+    oss_data *data = (oss_data*)device->ExtraData;
+    int numFragmentsLogSize;
+    int log2FragmentSize;
+    unsigned int periods;
+    audio_buf_info info;
+    ALuint frameSize;
+    int numChannels;
+    int ossFormat;
+    int ossSpeed;
+    char *err;
+    int i;
+
     switch(aluBytesFromFormat(device->Format))
     {
         case 1:
             ossFormat = AFMT_U8;
             break;
+        case 4:
+            switch(aluChannelsFromFormat(device->Format))
+            {
+                case 1: device->Format = AL_FORMAT_MONO16; break;
+                case 2: device->Format = AL_FORMAT_STEREO16; break;
+                case 4: device->Format = AL_FORMAT_QUAD16; break;
+                case 6: device->Format = AL_FORMAT_51CHN16; break;
+                case 7: device->Format = AL_FORMAT_61CHN16; break;
+                case 8: device->Format = AL_FORMAT_71CHN16; break;
+            }
+            /* fall-through */
         case 2:
             ossFormat = AFMT_S16_NE;
             break;
         default:
-            ossFormat = -1;
-            AL_PRINT("Unknown format?! %x\n", device->Format);
+            AL_PRINT("Unknown format: 0x%x\n", device->Format);
+            return ALC_FALSE;
     }
 
-    periods = GetConfigValueInt("oss", "periods", 4);
-    if((int)periods <= 0)
-        periods = 4;
+    periods = device->NumUpdates;
     numChannels = aluChannelsFromFormat(device->Format);
     frameSize = numChannels * aluBytesFromFormat(device->Format);
 
     ossSpeed = device->Frequency;
-    log2FragmentSize = log2i(device->UpdateSize * frameSize / periods);
+    log2FragmentSize = log2i(device->UpdateSize * frameSize);
 
     /* according to the OSS spec, 16 bytes are the minimum */
     if (log2FragmentSize < 4)
         log2FragmentSize = 4;
+    /* Subtract one period since the temp mixing buffer counts as one. Still
+     * need at least two on the card, though. */
+    if(periods > 2) periods--;
     numFragmentsLogSize = (periods << 16) | log2FragmentSize;
 
 #define ok(func, str) (i=(func),((i<0)?(err=(str)),0:1))
@@ -215,8 +244,6 @@ static ALCboolean oss_open_playback(ALCdevice *device, const ALCchar *deviceName
           ok(ioctl(data->fd, SNDCTL_DSP_GETOSPACE, &info), "get space")))
     {
         AL_PRINT("%s failed: %s\n", err, strerror(errno));
-        close(data->fd);
-        free(data);
         return ALC_FALSE;
     }
 #undef ok
@@ -224,8 +251,6 @@ static ALCboolean oss_open_playback(ALCdevice *device, const ALCchar *deviceName
     if((int)aluChannelsFromFormat(device->Format) != numChannels)
     {
         AL_PRINT("Could not set %d channels, got %d instead\n", aluChannelsFromFormat(device->Format), numChannels);
-        close(data->fd);
-        free(data);
         return ALC_FALSE;
     }
 
@@ -233,45 +258,44 @@ static ALCboolean oss_open_playback(ALCdevice *device, const ALCchar *deviceName
          (ossFormat == AFMT_S16_NE && aluBytesFromFormat(device->Format) == 2)))
     {
         AL_PRINT("Could not set %d-bit output, got format %#x\n", aluBytesFromFormat(device->Format)*8, ossFormat);
-        close(data->fd);
-        free(data);
-        return ALC_FALSE;
-    }
-
-    data->data_size = device->UpdateSize * frameSize;
-    data->mix_data = calloc(1, data->data_size);
-
-    device->ExtraData = data;
-    data->thread = StartThread(OSSProc, device);
-    if(data->thread == NULL)
-    {
-        device->ExtraData = NULL;
-        free(data->mix_data);
-        free(data);
         return ALC_FALSE;
     }
 
     device->Frequency = ossSpeed;
     device->UpdateSize = info.fragsize / frameSize;
+    device->NumUpdates = info.fragments + 1;
+
+    data->data_size = device->UpdateSize * frameSize;
+    data->mix_data = calloc(1, data->data_size);
+
+    data->thread = StartThread(OSSProc, device);
+    if(data->thread == NULL)
+    {
+        free(data->mix_data);
+        data->mix_data = NULL;
+        return ALC_FALSE;
+    }
 
     return ALC_TRUE;
 }
 
-static void oss_close_playback(ALCdevice *device)
+static void oss_stop_playback(ALCdevice *device)
 {
     oss_data *data = (oss_data*)device->ExtraData;
+
+    if(!data->thread)
+        return;
+
     data->killNow = 1;
     StopThread(data->thread);
-
-    close(data->fd);
+    data->thread = NULL;
 
     free(data->mix_data);
-    free(data);
-    device->ExtraData = NULL;
+    data->mix_data = NULL;
 }
 
 
-static ALCboolean oss_open_capture(ALCdevice *device, const ALCchar *deviceName, ALCuint frequency, ALCenum format, ALCsizei SampleSize)
+static ALCboolean oss_open_capture(ALCdevice *device, const ALCchar *deviceName)
 {
     int numFragmentsLogSize;
     int log2FragmentSize;
@@ -288,14 +312,10 @@ static ALCboolean oss_open_capture(ALCdevice *device, const ALCchar *deviceName,
 
     strncpy(driver, GetConfigValue("oss", "capture", "/dev/dsp"), sizeof(driver)-1);
     driver[sizeof(driver)-1] = 0;
-    if(deviceName)
-    {
-        if(strcmp(deviceName, oss_device_capture))
-            return ALC_FALSE;
-        device->szDeviceName = oss_device_capture;
-    }
-    else
-        device->szDeviceName = oss_device_capture;
+    if(!deviceName)
+        deviceName = oss_device_capture;
+    else if(strcmp(deviceName, oss_device_capture) != 0)
+        return ALC_FALSE;
 
     data = (oss_data*)calloc(1, sizeof(oss_data));
     data->killNow = 0;
@@ -308,7 +328,7 @@ static ALCboolean oss_open_capture(ALCdevice *device, const ALCchar *deviceName,
         return ALC_FALSE;
     }
 
-    switch(aluBytesFromFormat(format))
+    switch(aluBytesFromFormat(device->Format))
     {
         case 1:
             ossFormat = AFMT_U8;
@@ -317,15 +337,18 @@ static ALCboolean oss_open_capture(ALCdevice *device, const ALCchar *deviceName,
             ossFormat = AFMT_S16_NE;
             break;
         default:
-            ossFormat = -1;
-            AL_PRINT("Unknown format?! %x\n", device->Format);
+            AL_PRINT("Unknown format: 0x%x\n", device->Format);
+            close(data->fd);
+            free(data);
+            return ALC_FALSE;
     }
 
     periods = 4;
     numChannels = aluChannelsFromFormat(device->Format);
     frameSize = numChannels * aluBytesFromFormat(device->Format);
-    ossSpeed = frequency;
-    log2FragmentSize = log2i(SampleSize * frameSize / periods);
+    ossSpeed = device->Frequency;
+    log2FragmentSize = log2i(device->UpdateSize * device->NumUpdates *
+                             frameSize / periods);
 
     /* according to the OSS spec, 16 bytes are the minimum */
     if (log2FragmentSize < 4)
@@ -363,7 +386,7 @@ static ALCboolean oss_open_capture(ALCdevice *device, const ALCchar *deviceName,
         return ALC_FALSE;
     }
 
-    data->ring = CreateRingBuffer(frameSize, SampleSize);
+    data->ring = CreateRingBuffer(frameSize, device->UpdateSize * device->NumUpdates);
     if(!data->ring)
     {
         AL_PRINT("ring buffer create failed\n");
@@ -385,6 +408,7 @@ static ALCboolean oss_open_capture(ALCdevice *device, const ALCchar *deviceName,
         return ALC_FALSE;
     }
 
+    device->szDeviceName = strdup(deviceName);
     return ALC_TRUE;
 }
 
@@ -434,6 +458,8 @@ static ALCuint oss_available_samples(ALCdevice *pDevice)
 BackendFuncs oss_funcs = {
     oss_open_playback,
     oss_close_playback,
+    oss_reset_playback,
+    oss_stop_playback,
     oss_open_capture,
     oss_close_capture,
     oss_start_capture,
@@ -445,9 +471,18 @@ BackendFuncs oss_funcs = {
 void alc_oss_init(BackendFuncs *func_list)
 {
     *func_list = oss_funcs;
+}
 
-    oss_device = AppendDeviceList("OSS Software");
-    AppendAllDeviceList(oss_device);
+void alc_oss_deinit(void)
+{
+}
 
-    oss_device_capture = AppendCaptureDeviceList("OSS Capture");
+void alc_oss_probe(int type)
+{
+    if(type == DEVICE_PROBE)
+        AppendDeviceList(oss_device);
+    else if(type == ALL_DEVICE_PROBE)
+        AppendAllDeviceList(oss_device);
+    else if(type == CAPTURE_DEVICE_PROBE)
+        AppendCaptureDeviceList(oss_device_capture);
 }
