@@ -76,6 +76,7 @@ MAKE_FUNC(pa_threaded_mainloop_accept);
 MAKE_FUNC(pa_stream_set_write_callback);
 MAKE_FUNC(pa_threaded_mainloop_new);
 MAKE_FUNC(pa_context_connect);
+MAKE_FUNC(pa_stream_get_buffer_attr);
 MAKE_FUNC(pa_stream_set_read_callback);
 MAKE_FUNC(pa_stream_set_state_callback);
 MAKE_FUNC(pa_stream_new);
@@ -83,12 +84,12 @@ MAKE_FUNC(pa_stream_disconnect);
 MAKE_FUNC(pa_threaded_mainloop_lock);
 #undef MAKE_FUNC
 
-typedef struct {
-    ALCdevice *device;
+#ifndef PATH_MAX
+#define PATH_MAX 4096
+#endif
 
-    ALCenum format;
+typedef struct {
     ALCuint samples;
-    ALCuint frequency;
     ALCuint frame_size;
 
     RingBuffer *ring;
@@ -106,74 +107,21 @@ typedef struct {
     pa_context *context;
 } pulse_data;
 
-static char *pulse_device;
-static char *pulse_capture_device;
+static const ALCchar pulse_device[] = "PulseAudio Software";
+static const ALCchar pulse_capture_device[] = "PulseAudio Capture";
 
-// PulseAudio Event Callbacks {{{
-static void stream_state_callback(pa_stream *stream, void *pdata) //{{{
-{
-    pulse_data *data = pdata;
-
-    switch(ppa_stream_get_state(stream))
-	{
-		case PA_STREAM_READY:
-            AL_PRINT("%s: %s ready!\n", data->context_name, data->stream_name);
-			break;
-
-		case PA_STREAM_FAILED:
-            AL_PRINT("%s: %s: Connection failed: %s\n", data->context_name,
-                     data->stream_name, ppa_strerror(ppa_context_errno(data->context)));
-			break;
-
-		case PA_STREAM_TERMINATED:
-            AL_PRINT("%s: %s terminated!\n", data->context_name, data->stream_name);
-			break;
-
-		default:
-			break;
-	}
-
-    ppa_threaded_mainloop_signal(data->loop, 1);
-} //}}}
-
-static void context_state_callback(pa_context *context, void *pdata) //{{{
-{
-    pulse_data *data = pdata;
-
-    switch(ppa_context_get_state(context))
-	{
-		case PA_CONTEXT_READY:
-            AL_PRINT("%s ready!\n", data->context_name);
-			break;
-
-		case PA_CONTEXT_FAILED:
-            AL_PRINT("%s: Connection failed: %s\n", data->context_name,
-                     ppa_strerror(ppa_context_errno(context)));
-            break;
-
-		case PA_CONTEXT_TERMINATED:
-            AL_PRINT("%s terminated!\n", data->context_name);
-			break;
-
-		default:
-			break;
-	}
-
-    ppa_threaded_mainloop_signal(data->loop, 1);
-} //}}}
-//}}}
 
 // PulseAudio I/O Callbacks //{{{
 static void stream_write_callback(pa_stream *stream, size_t len, void *pdata) //{{{
 {
     ALCdevice *Device = pdata;
-    void *buf = ppa_xmalloc0(len);
+    pulse_data *data = Device->ExtraData;
+    void *buf = ppa_xmalloc0(data->attr.minreq);
+    (void)len;
 
-    SuspendContext(NULL);
-    aluMixData(Device->Context, buf, len, Device->Format);
-    ProcessContext(NULL);
-
-    ppa_stream_write(stream, buf, len, ppa_xfree, 0, PA_SEEK_RELATIVE);
+    aluMixData(Device, buf, data->attr.minreq/data->frame_size);
+    ppa_stream_write(stream, buf, data->attr.minreq, ppa_xfree, 0,
+                     PA_SEEK_RELATIVE);
 } //}}}
 
 static void stream_read_callback(pa_stream *stream, size_t length, void *pdata) //{{{
@@ -203,79 +151,26 @@ static void stream_read_callback(pa_stream *stream, size_t length, void *pdata)
 } //}}}
 //}}}
 
-static ALCboolean pulse_open(ALCdevice *device, ALCchar *device_name, ALCenum format, ALCuint samples, ALCuint frequency) //{{{
+static ALCboolean pulse_open(ALCdevice *device, const ALCchar *device_name) //{{{
 {
     pulse_data *data = ppa_xmalloc0(sizeof(pulse_data));
-
-    data->device = device;
-    data->format = format;
-    data->samples = samples;
-    data->frequency = frequency;
-    data->frame_size = aluBytesFromFormat(format) * aluChannelsFromFormat(format);
+    pa_context_state_t state;
 
     if(ppa_get_binary_name(data->path_name, sizeof(data->path_name)))
         data->context_name = ppa_path_get_filename(data->path_name);
     else
         data->context_name = "OpenAL Soft";
 
-    if(!(data->ring = CreateRingBuffer(data->frame_size, data->samples)))
-    {
-        ppa_xfree(data);
-        return ALC_FALSE;
-    }
-
-    device->ExtraData = data;
-    device->szDeviceName = device_name;
-
-    data->attr.minreq = -1;
-    data->attr.prebuf = -1;
-    data->attr.maxlength = -1;
-
-    if(device->IsCaptureDevice)
-    {
-        data->attr.tlength = -1;
-        data->attr.fragsize = data->frame_size * data->samples / 2;
-        data->stream_name = "Capture Stream";
-    }
-    else
-    {
-        data->attr.tlength = data->frame_size * (device->UpdateSize&~3);
-        data->attr.fragsize = -1;
-        data->stream_name = "Playback Stream";
-    }
-
-    data->spec.rate = data->frequency;
-    data->spec.channels = aluChannelsFromFormat(data->format);
-
-    switch(aluBytesFromFormat(data->format))
-    {
-        case 1:
-            data->spec.format = PA_SAMPLE_U8;
-            break;
-        case 2:
-            data->spec.format = PA_SAMPLE_S16NE;
-            break;
-        default:
-            AL_PRINT("Unknown format: %x\n", data->format);
-			goto out2;
-    }
-
-    if(ppa_sample_spec_valid(&data->spec) == 0)
-    {
-        AL_PRINT("Invalid sample format\n");
-		goto out2;
-    }
-
     if(!(data->loop = ppa_threaded_mainloop_new()))
     {
         AL_PRINT("pa_threaded_mainloop_new() failed!\n");
-		goto out2;
+        goto out;
     }
 
     if(ppa_threaded_mainloop_start(data->loop) < 0)
     {
         AL_PRINT("pa_threaded_mainloop_start() failed\n");
-		goto out3;
+        goto out;
     }
 
     ppa_threaded_mainloop_lock(data->loop);
@@ -287,115 +182,52 @@ static ALCboolean pulse_open(ALCdevice *device, ALCchar *device_name, ALCenum fo
                  ppa_strerror(ppa_context_errno(data->context)));
 
         ppa_threaded_mainloop_unlock(data->loop);
-		goto out3;
+        goto out;
     }
 
-    ppa_context_set_state_callback(data->context, context_state_callback, data);
-
     if(ppa_context_connect(data->context, NULL, PA_CONTEXT_NOAUTOSPAWN, NULL) < 0)
     {
         AL_PRINT("Context did not connect: %s\n",
                  ppa_strerror(ppa_context_errno(data->context)));
 
         ppa_context_unref(data->context);
-        ppa_threaded_mainloop_unlock(data->loop);
-
         data->context = NULL;
-		goto out3;
-    }
-
-    while(ppa_context_get_state(data->context) != PA_CONTEXT_READY)
-    {
-        if(!PA_CONTEXT_IS_GOOD(ppa_context_get_state(data->context)))
-        {
-            ppa_context_unref(data->context);
-            ppa_threaded_mainloop_unlock(data->loop);
-
-            data->context = NULL;
-			goto out3;
-        }
-
-        ppa_threaded_mainloop_wait(data->loop);
-        ppa_threaded_mainloop_accept(data->loop);
-    }
-
-    data->stream = ppa_stream_new(data->context, data->stream_name, &data->spec, NULL);
-    if(!data->stream)
-    {
-        AL_PRINT("pa_stream_new() failed: %s\n",
-                 ppa_strerror(ppa_context_errno(data->context)));
 
         ppa_threaded_mainloop_unlock(data->loop);
-		goto out4;
+        goto out;
     }
 
-    ppa_stream_set_state_callback(data->stream, stream_state_callback, data);
-
-    if(device->IsCaptureDevice)
+    while((state=ppa_context_get_state(data->context)) != PA_CONTEXT_READY)
     {
-        if(ppa_stream_connect_record(data->stream, NULL, &data->attr, PA_STREAM_ADJUST_LATENCY) < 0)
+        if(!PA_CONTEXT_IS_GOOD(state))
         {
-            AL_PRINT("Stream did not connect: %s\n",
+            AL_PRINT("Context did not get ready: %s\n",
                      ppa_strerror(ppa_context_errno(data->context)));
 
-            ppa_stream_unref(data->stream);
+            ppa_context_unref(data->context);
+            data->context = NULL;
+
             ppa_threaded_mainloop_unlock(data->loop);
-
-            data->stream = NULL;
-			goto out4;
-        }
-    }
-    else
-    {
-        ppa_stream_set_write_callback(data->stream, stream_write_callback, device);
-
-        if(ppa_stream_connect_playback(data->stream, NULL, &data->attr, PA_STREAM_ADJUST_LATENCY, NULL, NULL) < 0)
-        {
-            AL_PRINT("Stream did not connect: %s\n",
-                     ppa_strerror(ppa_context_errno(data->context)));
-
-            ppa_stream_unref(data->stream);
-            ppa_threaded_mainloop_unlock(data->loop);
-
-            data->stream = NULL;
-			goto out4;
-        }
-    }
-
-    while(ppa_stream_get_state(data->stream) != PA_STREAM_READY)
-    {
-        if(!PA_STREAM_IS_GOOD(ppa_stream_get_state(data->stream)))
-        {
-            ppa_stream_unref(data->stream);
-            ppa_threaded_mainloop_unlock(data->loop);
-
-            data->stream = NULL;
-			goto out4;
+            goto out;
         }
 
-        ppa_threaded_mainloop_wait(data->loop);
-        ppa_threaded_mainloop_accept(data->loop);
+        ppa_threaded_mainloop_unlock(data->loop);
+        Sleep(1);
+        ppa_threaded_mainloop_lock(data->loop);
     }
 
-    device->UpdateSize /= 4;
+    device->szDeviceName = strdup(device_name);
+    device->ExtraData = data;
+
     ppa_threaded_mainloop_unlock(data->loop);
-
     return ALC_TRUE;
 
-out4:
-    ppa_threaded_mainloop_lock(data->loop);
-
-    ppa_context_disconnect(data->context);
-    ppa_context_unref(data->context);
-
-    ppa_threaded_mainloop_unlock(data->loop);
-out3:
-    ppa_threaded_mainloop_stop(data->loop);
-    ppa_threaded_mainloop_free(data->loop);
-out2:
-    device->ExtraData = NULL;
-    device->szDeviceName = NULL;
-    DestroyRingBuffer(data->ring);
+out:
+    if(data->loop)
+    {
+        ppa_threaded_mainloop_stop(data->loop);
+        ppa_threaded_mainloop_free(data->loop);
+    }
 
     ppa_xfree(data);
     return ALC_FALSE;
@@ -407,8 +239,11 @@ static void pulse_close(ALCdevice *device) //{{{
 
     ppa_threaded_mainloop_lock(data->loop);
 
-    ppa_stream_disconnect(data->stream);
-    ppa_stream_unref(data->stream);
+    if(data->stream)
+    {
+        ppa_stream_disconnect(data->stream);
+        ppa_stream_unref(data->stream);
+    }
 
     ppa_context_disconnect(data->context);
     ppa_context_unref(data->context);
@@ -419,6 +254,7 @@ static void pulse_close(ALCdevice *device) //{{{
     ppa_threaded_mainloop_free(data->loop);
 
     device->ExtraData = NULL;
+    free(device->szDeviceName);
     device->szDeviceName = NULL;
     DestroyRingBuffer(data->ring);
 
@@ -432,13 +268,12 @@ static ALCboolean pulse_open_playback(ALCdevice *device, const ALCchar *device_n
     if(!pa_handle)
         return ALC_FALSE;
 
-    if(device_name)
-    {
-        if(strcmp(device_name, pulse_device) != 0)
-            return ALC_FALSE;
-    }
+    if(!device_name)
+        device_name = pulse_device;
+    else if(strcmp(device_name, pulse_device) != 0)
+        return ALC_FALSE;
 
-    return pulse_open(device, pulse_device, device->Format, 0, device->Frequency);
+    return pulse_open(device, device_name);
 } //}}}
 
 static void pulse_close_playback(ALCdevice *device) //{{{
@@ -446,18 +281,231 @@ static void pulse_close_playback(ALCdevice *device) //{{{
     pulse_close(device);
 } //}}}
 
-static ALCboolean pulse_open_capture(ALCdevice *device, const ALCchar *device_name, ALCuint frequency, ALCenum format, ALCsizei samples) //{{{
+static ALCboolean pulse_reset_playback(ALCdevice *device) //{{{
 {
+    pulse_data *data = device->ExtraData;
+    pa_stream_state_t state;
+
+    ppa_threaded_mainloop_lock(data->loop);
+
+    data->frame_size = aluBytesFromFormat(device->Format) *
+                       aluChannelsFromFormat(device->Format);
+    data->attr.minreq = data->frame_size * device->UpdateSize;
+    data->attr.prebuf = -1;
+    data->attr.maxlength = -1;
+    data->attr.fragsize = -1;
+    data->attr.tlength = data->attr.minreq * device->NumUpdates;
+    data->stream_name = "Playback Stream";
+
+    switch(aluBytesFromFormat(device->Format))
+    {
+        case 1:
+            data->spec.format = PA_SAMPLE_U8;
+            break;
+        case 2:
+            data->spec.format = PA_SAMPLE_S16NE;
+            break;
+        case 4:
+            data->spec.format = PA_SAMPLE_FLOAT32NE;
+            break;
+        default:
+            AL_PRINT("Unknown format: 0x%x\n", device->Format);
+            ppa_threaded_mainloop_unlock(data->loop);
+            return ALC_FALSE;
+    }
+    data->spec.rate = device->Frequency;
+    data->spec.channels = aluChannelsFromFormat(device->Format);
+
+    if(ppa_sample_spec_valid(&data->spec) == 0)
+    {
+        AL_PRINT("Invalid sample format\n");
+        ppa_threaded_mainloop_unlock(data->loop);
+        return ALC_FALSE;
+    }
+
+    data->stream = ppa_stream_new(data->context, data->stream_name, &data->spec, NULL);
+    if(!data->stream)
+    {
+        AL_PRINT("pa_stream_new() failed: %s\n",
+                 ppa_strerror(ppa_context_errno(data->context)));
+
+        ppa_threaded_mainloop_unlock(data->loop);
+        return ALC_FALSE;
+    }
+
+    if(ppa_stream_connect_playback(data->stream, NULL, &data->attr, PA_STREAM_ADJUST_LATENCY, NULL, NULL) < 0)
+    {
+        AL_PRINT("Stream did not connect: %s\n",
+                 ppa_strerror(ppa_context_errno(data->context)));
+
+        ppa_stream_unref(data->stream);
+        data->stream = NULL;
+
+        ppa_threaded_mainloop_unlock(data->loop);
+        return ALC_FALSE;
+    }
+
+    while((state=ppa_stream_get_state(data->stream)) != PA_STREAM_READY)
+    {
+        if(!PA_STREAM_IS_GOOD(state))
+        {
+            AL_PRINT("Stream did not get ready: %s\n",
+                     ppa_strerror(ppa_context_errno(data->context)));
+
+            ppa_stream_unref(data->stream);
+            data->stream = NULL;
+
+            ppa_threaded_mainloop_unlock(data->loop);
+            return ALC_FALSE;
+        }
+
+        ppa_threaded_mainloop_unlock(data->loop);
+        Sleep(1);
+        ppa_threaded_mainloop_lock(data->loop);
+    }
+
+    data->attr = *(ppa_stream_get_buffer_attr(data->stream));
+    if((data->attr.tlength%data->attr.minreq) != 0)
+        AL_PRINT("tlength (%d) is not a multiple of minreq (%d)!\n",
+                 data->attr.tlength, data->attr.minreq);
+    device->UpdateSize = data->attr.minreq;
+    device->NumUpdates = data->attr.tlength/data->attr.minreq;
+
+    ppa_stream_set_write_callback(data->stream, stream_write_callback, device);
+
+    ppa_threaded_mainloop_unlock(data->loop);
+    return ALC_TRUE;
+} //}}}
+
+static void pulse_stop_playback(ALCdevice *device) //{{{
+{
+    pulse_data *data = device->ExtraData;
+
+    if(!data->stream)
+        return;
+
+    ppa_threaded_mainloop_lock(data->loop);
+
+    ppa_stream_disconnect(data->stream);
+    ppa_stream_unref(data->stream);
+    data->stream = NULL;
+
+    ppa_threaded_mainloop_unlock(data->loop);
+} //}}}
+
+
+static ALCboolean pulse_open_capture(ALCdevice *device, const ALCchar *device_name) //{{{
+{
+    pulse_data *data;
+    pa_stream_state_t state;
+
     if(!pa_handle)
         return ALC_FALSE;
 
-    if(device_name)
+    if(!device_name)
+        device_name = pulse_capture_device;
+    else if(strcmp(device_name, pulse_capture_device) != 0)
+        return ALC_FALSE;
+
+    if(pulse_open(device, device_name) == ALC_FALSE)
+        return ALC_FALSE;
+
+    data = device->ExtraData;
+    ppa_threaded_mainloop_lock(data->loop);
+
+    data->samples = device->UpdateSize * device->NumUpdates;
+    data->frame_size = aluBytesFromFormat(device->Format) *
+                       aluChannelsFromFormat(device->Format);
+
+    if(!(data->ring = CreateRingBuffer(data->frame_size, data->samples)))
     {
-        if(strcmp(device_name, pulse_capture_device) != 0)
+        ppa_threaded_mainloop_unlock(data->loop);
+        pulse_close(device);
+        return ALC_FALSE;
+    }
+
+    data->attr.minreq = -1;
+    data->attr.prebuf = -1;
+    data->attr.maxlength = -1;
+    data->attr.tlength = -1;
+    data->attr.fragsize = data->frame_size * data->samples / 2;
+    data->stream_name = "Capture Stream";
+
+    data->spec.rate = device->Frequency;
+    data->spec.channels = aluChannelsFromFormat(device->Format);
+
+    switch(aluBytesFromFormat(device->Format))
+    {
+        case 1:
+            data->spec.format = PA_SAMPLE_U8;
+            break;
+        case 2:
+            data->spec.format = PA_SAMPLE_S16NE;
+            break;
+        case 4:
+            data->spec.format = PA_SAMPLE_FLOAT32NE;
+            break;
+        default:
+            AL_PRINT("Unknown format: 0x%x\n", device->Format);
+            ppa_threaded_mainloop_unlock(data->loop);
+            pulse_close(device);
             return ALC_FALSE;
     }
 
-    return pulse_open(device, pulse_capture_device, format, samples, frequency);
+    if(ppa_sample_spec_valid(&data->spec) == 0)
+    {
+        AL_PRINT("Invalid sample format\n");
+        ppa_threaded_mainloop_unlock(data->loop);
+        pulse_close(device);
+        return ALC_FALSE;
+    }
+
+    data->stream = ppa_stream_new(data->context, data->stream_name, &data->spec, NULL);
+    if(!data->stream)
+    {
+        AL_PRINT("pa_stream_new() failed: %s\n",
+                 ppa_strerror(ppa_context_errno(data->context)));
+
+        ppa_threaded_mainloop_unlock(data->loop);
+        pulse_close(device);
+        return ALC_FALSE;
+    }
+
+    if(ppa_stream_connect_record(data->stream, NULL, &data->attr, PA_STREAM_ADJUST_LATENCY) < 0)
+    {
+        AL_PRINT("Stream did not connect: %s\n",
+                 ppa_strerror(ppa_context_errno(data->context)));
+
+        ppa_stream_unref(data->stream);
+        ppa_threaded_mainloop_unlock(data->loop);
+
+        data->stream = NULL;
+        pulse_close(device);
+        return ALC_FALSE;
+    }
+
+    while((state=ppa_stream_get_state(data->stream)) != PA_STREAM_READY)
+    {
+        if(!PA_STREAM_IS_GOOD(state))
+        {
+            AL_PRINT("Stream did not get ready: %s\n",
+                     ppa_strerror(ppa_context_errno(data->context)));
+
+            ppa_stream_unref(data->stream);
+            ppa_threaded_mainloop_unlock(data->loop);
+
+            data->stream = NULL;
+            pulse_close(device);
+            return ALC_FALSE;
+        }
+
+        ppa_threaded_mainloop_unlock(data->loop);
+        Sleep(1);
+        ppa_threaded_mainloop_lock(data->loop);
+    }
+
+    ppa_threaded_mainloop_unlock(data->loop);
+    return ALC_TRUE;
 } //}}}
 
 static void pulse_close_capture(ALCdevice *device) //{{{
@@ -503,6 +551,8 @@ static ALCuint pulse_available_samples(ALCdevice *device) //{{{
 BackendFuncs pulse_funcs = { //{{{
     pulse_open_playback,
     pulse_close_playback,
+    pulse_reset_playback,
+    pulse_stop_playback,
     pulse_open_capture,
     pulse_close_capture,
     pulse_start_capture,
@@ -513,13 +563,14 @@ BackendFuncs pulse_funcs = { //{{{
 
 void alc_pulse_init(BackendFuncs *func_list) //{{{
 {
-    *func_list = pulse_funcs;
+    if(func_list) *func_list = pulse_funcs;
 
 #ifdef _WIN32
     pa_handle = LoadLibrary("libpulse-0.dll");
 #define LOAD_FUNC(x) do { \
     p##x = GetProcAddress(pa_handle, #x); \
     if(!(p##x)) { \
+        AL_PRINT("Could not load %s from libpulse-0.dll\n", #x); \
         FreeLibrary(pa_handle); \
         pa_handle = NULL; \
         return; \
@@ -536,6 +587,7 @@ void alc_pulse_init(BackendFuncs *func_list) //{{{
 #define LOAD_FUNC(x) do { \
     p##x = dlsym(pa_handle, #x); \
     if(!(p##x)) { \
+        AL_PRINT("Could not load %s from libpulse\n", #x); \
         dlclose(pa_handle); \
         pa_handle = NULL; \
         return; \
@@ -548,6 +600,8 @@ void alc_pulse_init(BackendFuncs *func_list) //{{{
 #define LOAD_FUNC(x) p##x = (x)
 
 #endif
+    if(!pa_handle)
+        return;
 
 LOAD_FUNC(pa_context_unref);
 LOAD_FUNC(pa_sample_spec_valid);
@@ -579,6 +633,7 @@ LOAD_FUNC(pa_threaded_mainloop_accept);
 LOAD_FUNC(pa_stream_set_write_callback);
 LOAD_FUNC(pa_threaded_mainloop_new);
 LOAD_FUNC(pa_context_connect);
+LOAD_FUNC(pa_stream_get_buffer_attr);
 LOAD_FUNC(pa_stream_set_read_callback);
 LOAD_FUNC(pa_stream_set_state_callback);
 LOAD_FUNC(pa_stream_new);
@@ -586,10 +641,31 @@ LOAD_FUNC(pa_stream_disconnect);
 LOAD_FUNC(pa_threaded_mainloop_lock);
 
 #undef LOAD_FUNC
+} //}}}
 
-    pulse_device = AppendDeviceList("PulseAudio Software");
-    AppendAllDeviceList(pulse_device);
+void alc_pulse_deinit(void) //{{{
+{
+    if(pa_handle)
+    {
+#ifdef _WIN32
+        FreeLibrary(pa_handle);
+#elif defined (HAVE_DLFCN_H)
+        dlclose(pa_handle);
+#endif
+    }
+    pa_handle = NULL;
+} //}}}
 
-    pulse_capture_device = AppendCaptureDeviceList("PulseAudio Capture");
+void alc_pulse_probe(int type) //{{{
+{
+    if(!pa_handle) alc_pulse_init(NULL);
+    if(!pa_handle) return;
+
+    if(type == DEVICE_PROBE)
+        AppendDeviceList(pulse_device);
+    else if(type == ALL_DEVICE_PROBE)
+        AppendAllDeviceList(pulse_device);
+    else if(type == CAPTURE_DEVICE_PROBE)
+        AppendCaptureDeviceList(pulse_capture_device);
 } //}}}
 //}}}
