@@ -39,17 +39,12 @@ ALvoid AL_APIENTRY alGenAuxiliaryEffectSlots(ALsizei n, ALuint *effectslots)
     ALCcontext *Context;
     ALsizei i, j;
 
-    Context = alcGetCurrentContext();
-    if(!Context)
-    {
-        alSetError(AL_INVALID_OPERATION);
-        return;
-    }
-    SuspendContext(Context);
+    Context = GetContextSuspended();
+    if(!Context) return;
 
     if (n > 0)
     {
-        ALCdevice *Device = alcGetContextsDevice(Context);
+        ALCdevice *Device = Context->Device;
 
         if(Context->AuxiliaryEffectSlotCount+n <= Device->AuxiliaryEffectSlotMax)
         {
@@ -101,13 +96,8 @@ ALvoid AL_APIENTRY alDeleteAuxiliaryEffectSlots(ALsizei n, ALuint *effectslots)
     ALeffectslot *ALAuxiliaryEffectSlot;
     ALsizei i;
 
-    Context = alcGetCurrentContext();
-    if(!Context)
-    {
-        alSetError(AL_INVALID_OPERATION);
-        return;
-    }
-    SuspendContext(Context);
+    Context = GetContextSuspended();
+    if(!Context) return;
 
     if (n >= 0)
     {
@@ -173,13 +163,8 @@ ALboolean AL_APIENTRY alIsAuxiliaryEffectSlot(ALuint effectslot)
     ALCcontext *Context;
     ALeffectslot **list;
 
-    Context = alcGetCurrentContext();
-    if(!Context)
-    {
-        alSetError(AL_INVALID_OPERATION);
-        return AL_FALSE;
-    }
-    SuspendContext(Context);
+    Context = GetContextSuspended();
+    if(!Context) return AL_FALSE;
 
     list = &Context->AuxiliaryEffectSlot;
     while(*list && (*list)->effectslot != effectslot)
@@ -194,13 +179,8 @@ ALvoid AL_APIENTRY alAuxiliaryEffectSloti(ALuint effectslot, ALenum param, ALint
 {
     ALCcontext *Context;
 
-    Context = alcGetCurrentContext();
-    if(!Context)
-    {
-        alSetError(AL_INVALID_OPERATION);
-        return;
-    }
-    SuspendContext(Context);
+    Context = GetContextSuspended();
+    if(!Context) return;
 
     if (alIsAuxiliaryEffectSlot(effectslot))
     {
@@ -240,13 +220,8 @@ ALvoid AL_APIENTRY alAuxiliaryEffectSlotiv(ALuint effectslot, ALenum param, ALin
 {
     ALCcontext *Context;
 
-    Context = alcGetCurrentContext();
-    if(!Context)
-    {
-        alSetError(AL_INVALID_OPERATION);
-        return;
-    }
-    SuspendContext(Context);
+    Context = GetContextSuspended();
+    if(!Context) return;
 
     if (alIsAuxiliaryEffectSlot(effectslot))
     {
@@ -272,13 +247,8 @@ ALvoid AL_APIENTRY alAuxiliaryEffectSlotf(ALuint effectslot, ALenum param, ALflo
 {
     ALCcontext *Context;
 
-    Context = alcGetCurrentContext();
-    if(!Context)
-    {
-        alSetError(AL_INVALID_OPERATION);
-        return;
-    }
-    SuspendContext(Context);
+    Context = GetContextSuspended();
+    if(!Context) return;
 
     if (alIsAuxiliaryEffectSlot(effectslot))
     {
@@ -308,13 +278,8 @@ ALvoid AL_APIENTRY alAuxiliaryEffectSlotfv(ALuint effectslot, ALenum param, ALfl
 {
     ALCcontext *Context;
 
-    Context = alcGetCurrentContext();
-    if(!Context)
-    {
-        alSetError(AL_INVALID_OPERATION);
-        return;
-    }
-    SuspendContext(Context);
+    Context = GetContextSuspended();
+    if(!Context) return;
 
     if (alIsAuxiliaryEffectSlot(effectslot))
     {
@@ -339,13 +304,8 @@ ALvoid AL_APIENTRY alGetAuxiliaryEffectSloti(ALuint effectslot, ALenum param, AL
 {
     ALCcontext *Context;
 
-    Context = alcGetCurrentContext();
-    if(!Context)
-    {
-        alSetError(AL_INVALID_OPERATION);
-        return;
-    }
-    SuspendContext(Context);
+    Context = GetContextSuspended();
+    if(!Context) return;
 
     if (alIsAuxiliaryEffectSlot(effectslot))
     {
@@ -376,13 +336,8 @@ ALvoid AL_APIENTRY alGetAuxiliaryEffectSlotiv(ALuint effectslot, ALenum param, A
 {
     ALCcontext *Context;
 
-    Context = alcGetCurrentContext();
-    if(!Context)
-    {
-        alSetError(AL_INVALID_OPERATION);
-        return;
-    }
-    SuspendContext(Context);
+    Context = GetContextSuspended();
+    if(!Context) return;
 
     if (alIsAuxiliaryEffectSlot(effectslot))
     {
@@ -408,13 +363,8 @@ ALvoid AL_APIENTRY alGetAuxiliaryEffectSlotf(ALuint effectslot, ALenum param, AL
 {
     ALCcontext *Context;
 
-    Context = alcGetCurrentContext();
-    if(!Context)
-    {
-        alSetError(AL_INVALID_OPERATION);
-        return;
-    }
-    SuspendContext(Context);
+    Context = GetContextSuspended();
+    if(!Context) return;
 
     if (alIsAuxiliaryEffectSlot(effectslot))
     {
@@ -441,13 +391,8 @@ ALvoid AL_APIENTRY alGetAuxiliaryEffectSlotfv(ALuint effectslot, ALenum param, A
 {
     ALCcontext *Context;
 
-    Context = alcGetCurrentContext();
-    if(!Context)
-    {
-        alSetError(AL_INVALID_OPERATION);
-        return;
-    }
-    SuspendContext(Context);
+    Context = GetContextSuspended();
+    if(!Context) return;
 
     if (alIsAuxiliaryEffectSlot(effectslot))
     {
@@ -502,11 +447,6 @@ static ALvoid InitializeEffect(ALCcontext *Context, ALeffectslot *ALEffectSlot,
 
 ALvoid ReleaseALAuxiliaryEffectSlots(ALCcontext *Context)
 {
-#ifdef _DEBUG
-    if(Context->AuxiliaryEffectSlotCount > 0)
-        AL_PRINT("alcDestroyContext(): deleting %d AuxiliaryEffectSlot(s)\n", Context->AuxiliaryEffectSlotCount);
-#endif
-
     while(Context->AuxiliaryEffectSlot)
     {
         ALeffectslot *temp = Context->AuxiliaryEffectSlot;
