@@ -28,6 +28,7 @@
 #include "AL/alc.h"
 #include "alError.h"
 #include "alBuffer.h"
+#include "alDatabuffer.h"
 #include "alThunk.h"
 
 
@@ -49,9 +50,6 @@ static void ConvertDataIMA4(ALshort *dst, const ALvoid *src, ALint origChans, AL
 * Global Variables
 */
 
-static ALbuffer *g_pBuffers = NULL;          // Linked List of Buffers
-static ALuint    g_uiBufferCount = 0;        // Buffer Count
-
 static const long g_IMAStep_size[89]={            // IMA ADPCM Stepsize table
        7,    8,    9,   10,   11,   12,   13,   14,   16,   17,   19,   21,   23,   25,   28,   31,
       34,   37,   41,   45,   50,   55,   60,   66,   73,   80,   88,   97,  107,  118,  130,  143,
@@ -81,16 +79,18 @@ ALAPI ALvoid ALAPIENTRY alGenBuffers(ALsizei n,ALuint *puiBuffers)
     ALCcontext *Context;
     ALsizei i=0;
 
-    Context = alcGetCurrentContext();
-    SuspendContext(Context);
+    Context = GetContextSuspended();
+    if(!Context) return;
 
     // Check that we are actually generation some Buffers
     if (n > 0)
     {
+        ALCdevice *device = Context->Device;
+
         // Check the pointer is valid (and points to enough memory to store Buffer Names)
         if (!IsBadWritePtr((void*)puiBuffers, n * sizeof(ALuint)))
         {
-            ALbuffer **list = &g_pBuffers;
+            ALbuffer **list = &device->Buffers;
             while(*list)
                 list = &(*list)->next;
 
@@ -107,7 +107,7 @@ ALAPI ALvoid ALAPIENTRY alGenBuffers(ALsizei n,ALuint *puiBuffers)
 
                 puiBuffers[i] = (ALuint)ALTHUNK_ADDENTRY(*list);
                 (*list)->state = UNUSED;
-                g_uiBufferCount++;
+                device->BufferCount++;
                 i++;
 
                 list = &(*list)->next;
@@ -137,12 +137,14 @@ ALAPI ALvoid ALAPIENTRY alDeleteBuffers(ALsizei n, const ALuint *puiBuffers)
     ALsizei i;
     ALboolean bFailed = AL_FALSE;
 
-    Context = alcGetCurrentContext();
-    SuspendContext(Context);
+    Context = GetContextSuspended();
+    if(!Context) return;
 
     // Check we are actually Deleting some Buffers
     if (n >= 0)
     {
+        ALCdevice *device = Context->Device;
+
         // Check that all the buffers are valid and can actually be deleted
         for (i = 0; i < n; i++)
         {
@@ -176,7 +178,7 @@ ALAPI ALvoid ALAPIENTRY alDeleteBuffers(ALsizei n, const ALuint *puiBuffers)
             {
                 if (puiBuffers[i] && alIsBuffer(puiBuffers[i]))
                 {
-                    ALbuffer **list = &g_pBuffers;
+                    ALbuffer **list = &device->Buffers;
 
                     ALBuf=((ALbuffer *)ALTHUNK_LOOKUPENTRY(puiBuffers[i]));
                     while(*list && *list != ALBuf)
@@ -191,7 +193,7 @@ ALAPI ALvoid ALAPIENTRY alDeleteBuffers(ALsizei n, const ALuint *puiBuffers)
                     // Release buffer structure
                     ALTHUNK_REMOVEENTRY(puiBuffers[i]);
                     memset(ALBuf, 0, sizeof(ALbuffer));
-                    g_uiBufferCount--;
+                    device->BufferCount--;
                     free(ALBuf);
                 }
             }
@@ -218,15 +220,17 @@ ALAPI ALboolean ALAPIENTRY alIsBuffer(ALuint uiBuffer)
     ALbuffer *ALBuf;
     ALbuffer *TgtALBuf;
 
-    Context = alcGetCurrentContext();
-    SuspendContext(Context);
+    Context = GetContextSuspended();
+    if(!Context) return AL_FALSE;
 
     if (uiBuffer)
     {
+        ALCdevice *device = Context->Device;
+
         TgtALBuf = (ALbuffer *)ALTHUNK_LOOKUPENTRY(uiBuffer);
 
         // Check through list of generated buffers for uiBuffer
-        ALBuf = g_pBuffers;
+        ALBuf = device->Buffers;
         while (ALBuf)
         {
             if (ALBuf == TgtALBuf)
@@ -261,12 +265,28 @@ ALAPI ALvoid ALAPIENTRY alBufferData(ALuint buffer,ALenum format,const ALvoid *d
     ALbuffer *ALBuf;
     ALvoid *temp;
 
-    Context = alcGetCurrentContext();
-    SuspendContext(Context);
+    Context = GetContextSuspended();
+    if(!Context) return;
 
     if (alIsBuffer(buffer) && (buffer != 0))
     {
         ALBuf=((ALbuffer *)ALTHUNK_LOOKUPENTRY(buffer));
+
+        if(Context->SampleSource)
+        {
+            ALuint offset;
+
+            if(Context->SampleSource->state == MAPPED)
+            {
+                alSetError(AL_INVALID_OPERATION);
+                ProcessContext(Context);
+                return;
+            }
+
+            offset = (ALuint)data;
+            data = Context->SampleSource->data + offset;
+        }
+
         if ((ALBuf->refcount==0)&&(data))
         {
             switch(format)
@@ -413,12 +433,28 @@ ALvoid ALAPIENTRY alBufferSubDataEXT(ALuint buffer,ALenum format,const ALvoid *d
     ALCcontext *Context;
     ALbuffer *ALBuf;
 
-    Context = alcGetCurrentContext();
-    SuspendContext(Context);
+    Context = GetContextSuspended();
+    if(!Context) return;
 
     if(alIsBuffer(buffer) && buffer != 0)
     {
         ALBuf = (ALbuffer*)ALTHUNK_LOOKUPENTRY(buffer);
+
+        if(Context->SampleSource)
+        {
+            ALuint offset;
+
+            if(Context->SampleSource->state == MAPPED)
+            {
+                alSetError(AL_INVALID_OPERATION);
+                ProcessContext(Context);
+                return;
+            }
+
+            offset = (ALuint)data;
+            data = Context->SampleSource->data + offset;
+        }
+
         if(ALBuf->data == NULL)
         {
             // buffer does not have any data
@@ -514,8 +550,8 @@ ALAPI void ALAPIENTRY alBufferf(ALuint buffer, ALenum eParam, ALfloat flValue)
 
     (void)flValue;
 
-    pContext = alcGetCurrentContext();
-    SuspendContext(pContext);
+    pContext = GetContextSuspended();
+    if(!pContext) return;
 
     if (alIsBuffer(buffer) && (buffer != 0))
     {
@@ -543,8 +579,8 @@ ALAPI void ALAPIENTRY alBuffer3f(ALuint buffer, ALenum eParam, ALfloat flValue1,
     (void)flValue2;
     (void)flValue3;
 
-    pContext = alcGetCurrentContext();
-    SuspendContext(pContext);
+    pContext = GetContextSuspended();
+    if(!pContext) return;
 
     if (alIsBuffer(buffer) && (buffer != 0))
     {
@@ -570,8 +606,8 @@ ALAPI void ALAPIENTRY alBufferfv(ALuint buffer, ALenum eParam, const ALfloat* fl
 
     (void)flValues;
 
-    pContext = alcGetCurrentContext();
-    SuspendContext(pContext);
+    pContext = GetContextSuspended();
+    if(!pContext) return;
 
     if (alIsBuffer(buffer) && (buffer != 0))
     {
@@ -597,8 +633,8 @@ ALAPI void ALAPIENTRY alBufferi(ALuint buffer, ALenum eParam, ALint lValue)
 
     (void)lValue;
 
-    pContext = alcGetCurrentContext();
-    SuspendContext(pContext);
+    pContext = GetContextSuspended();
+    if(!pContext) return;
 
     if (alIsBuffer(buffer) && (buffer != 0))
     {
@@ -626,8 +662,8 @@ ALAPI void ALAPIENTRY alBuffer3i( ALuint buffer, ALenum eParam, ALint lValue1, A
     (void)lValue2;
     (void)lValue3;
 
-    pContext = alcGetCurrentContext();
-    SuspendContext(pContext);
+    pContext = GetContextSuspended();
+    if(!pContext) return;
 
     if (alIsBuffer(buffer) && (buffer != 0))
     {
@@ -653,8 +689,8 @@ ALAPI void ALAPIENTRY alBufferiv(ALuint buffer, ALenum eParam, const ALint* plVa
 
     (void)plValues;
 
-    pContext = alcGetCurrentContext();
-    SuspendContext(pContext);
+    pContext = GetContextSuspended();
+    if(!pContext) return;
 
     if (alIsBuffer(buffer) && (buffer != 0))
     {
@@ -678,8 +714,8 @@ ALAPI ALvoid ALAPIENTRY alGetBufferf(ALuint buffer, ALenum eParam, ALfloat *pflV
 {
     ALCcontext    *pContext;
 
-    pContext = alcGetCurrentContext();
-    SuspendContext(pContext);
+    pContext = GetContextSuspended();
+    if(!pContext) return;
 
     if (pflValue)
     {
@@ -710,8 +746,8 @@ ALAPI void ALAPIENTRY alGetBuffer3f(ALuint buffer, ALenum eParam, ALfloat* pflVa
 {
     ALCcontext    *pContext;
 
-    pContext = alcGetCurrentContext();
-    SuspendContext(pContext);
+    pContext = GetContextSuspended();
+    if(!pContext) return;
 
     if ((pflValue1) && (pflValue2) && (pflValue3))
     {
@@ -742,8 +778,8 @@ ALAPI void ALAPIENTRY alGetBufferfv(ALuint buffer, ALenum eParam, ALfloat* pflVa
 {
     ALCcontext    *pContext;
 
-    pContext = alcGetCurrentContext();
-    SuspendContext(pContext);
+    pContext = GetContextSuspended();
+    if(!pContext) return;
 
     if (pflValues)
     {
@@ -775,8 +811,8 @@ ALAPI ALvoid ALAPIENTRY alGetBufferi(ALuint buffer, ALenum eParam, ALint *plValu
     ALCcontext    *pContext;
     ALbuffer    *pBuffer;
 
-    pContext = alcGetCurrentContext();
-    SuspendContext(pContext);
+    pContext = GetContextSuspended();
+    if(!pContext) return;
 
     if (plValue)
     {
@@ -825,8 +861,8 @@ ALAPI void ALAPIENTRY alGetBuffer3i(ALuint buffer, ALenum eParam, ALint* plValue
 {
     ALCcontext    *pContext;
 
-    pContext = alcGetCurrentContext();
-    SuspendContext(pContext);
+    pContext = GetContextSuspended();
+    if(!pContext) return;
 
     if ((plValue1) && (plValue2) && (plValue3))
     {
@@ -857,8 +893,8 @@ ALAPI void ALAPIENTRY alGetBufferiv(ALuint buffer, ALenum eParam, ALint* plValue
 {
     ALCcontext    *pContext;
 
-    pContext = alcGetCurrentContext();
-    SuspendContext(pContext);
+    pContext = GetContextSuspended();
+    if(!pContext) return;
 
     if (plValues)
     {
@@ -1069,17 +1105,12 @@ static void ConvertDataIMA4(ALshort *dst, const ALvoid *src, ALint origChans, AL
 *
 *    INTERNAL FN : Called by DLLMain on exit to destroy any buffers that still exist
 */
-ALvoid ReleaseALBuffers(ALvoid)
+ALvoid ReleaseALBuffers(ALCdevice *device)
 {
     ALbuffer *ALBuffer;
     ALbuffer *ALBufferTemp;
 
-#ifdef _DEBUG
-    if(g_uiBufferCount > 0)
-        AL_PRINT("exit(): deleting %d Buffer(s)\n", g_uiBufferCount);
-#endif
-
-    ALBuffer = g_pBuffers;
+    ALBuffer = device->Buffers;
     while(ALBuffer)
     {
         // Release sample data
@@ -1091,6 +1122,6 @@ ALvoid ReleaseALBuffers(ALvoid)
         memset(ALBufferTemp, 0, sizeof(ALbuffer));
         free(ALBufferTemp);
     }
-    g_pBuffers = NULL;
-    g_uiBufferCount = 0;
+    device->Buffers = NULL;
+    device->BufferCount = 0;
 }
